// @ts-check

const http = require("http");
const https = require("https");
const path = require("path");
const fs = require("fs");
const grabHostNames = require("../package-shared/utils/grab-host-names");
const apiUpdateUser = require("../package-shared/functions/api/users/api-update-user");
const apiDeleteUser = require("../package-shared/functions/api/users/api-delete-user");

/**
 * # Update User
 * @async
 *
 * @param {object} params - API Key
 * @param {String} [params.key] - API Key
 * @param {String} [params.database] - Target Database
 * @param {String | number} params.deletedUserId - Target Database
 * @param {boolean} [params.user_id] - User ID
 * @param {boolean} [params.useLocal]
 *
 * @returns { Promise<import("../package-shared/types").UpdateUserFunctionReturn>}
 */
async function deleteUser({ key, database, user_id, useLocal, deletedUserId }) {
    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } =
        process.env;

    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        useLocal
    ) {
        /** @type {import("../package-shared/types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        return await apiDeleteUser({
            dbFullName: DSQL_DB_NAME,
            useLocal,
            deletedUserId,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        const reqPayload = JSON.stringify({
            database,
            deletedUserId,
        });

        const httpsRequest = scheme.request(
            {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization:
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY ||
                        key,
                },
                port,
                hostname: host,
                path: `/api/user/${
                    user_id || grabedHostNames.user_id
                }/delete-user`,
            },

            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";

                response.on("data", function (chunk) {
                    str += chunk;
                });

                response.on("end", function () {
                    resolve(JSON.parse(str));
                });

                response.on("error", (err) => {
                    reject(err);
                });
            }
        );
        httpsRequest.write(reqPayload);
        httpsRequest.end();
    });

    /** ********************************************** */
    /** ********************************************** */
    /** ********************************************** */

    return httpResponse;
}

/** ********************************************** */
/** ********************************************** */
/** ********************************************** */

module.exports = deleteUser;
